import logging
import sys
from distutils.command.build_ext import build_ext
from distutils.core import Extension
from distutils.errors import CCompilerError
from distutils.errors import DistutilsExecError
from distutils.errors import DistutilsPlatformError

log = logging.getLogger(__name__)

args = []
if sys.platform == "win32":
    args.append("/Ox")
else:
    args.append("-O3")

ext_modules = [
    Extension("_case", sources=["src/yascc/case.c"], extra_compile_args=args),
]


class BuildFailed(Exception):
    pass


class ExtBuilder(build_ext):
    def run(self):
        try:
            build_ext.run(self)
        except (DistutilsPlatformError, FileNotFoundError) as e:
            log.error("Could not compile C extension.")
            raise e from None

    def build_extension(self, ext):
        try:
            build_ext.build_extension(self, ext)
        except (
            CCompilerError,
            DistutilsExecError,
            DistutilsPlatformError,
            ValueError,
        ) as e:
            log.error("Could not compile C extension.")
            raise e from None


def build(setup_kwargs):
    """
    This function is mandatory in order to build the extensions.
    """
    setup_kwargs.update({"ext_modules": ext_modules, "cmdclass": {"build_ext": ExtBuilder}})



import json

import pytest

from yascc import snakecase

from .django_rest_framework_camelcase import camelize


def test_camelize_dict(camelized_dict, decamelized_dict) -> None:
    got = snakecase.camelize(decamelized_dict)
    assert got == camelized_dict


def test_camelize_nested_dict(decamelized_nested_dict, camelized_nested_dict) -> None:
    assert snakecase.camelize(decamelized_nested_dict) == camelized_nested_dict


@pytest.mark.parametrize(
    "arg,expected",
    [
        (None, None),
        ([None], [None]),
        ([{None: None}], [{None: None}]),
        ([{"some_value": None}], [{"someValue": None}]),
    ],
)
def test_camelize(arg, expected):
    assert snakecase.camelize(arg) == expected


@pytest.mark.parametrize("impl", [snakecase.camelize, camelize], ids=["c", "drf"])
def test_camelize_list_benchmark(benchmark, decamelized_list_of_dict, impl):
    benchmark(impl, decamelized_list_of_dict)


@pytest.mark.parametrize("impl", [snakecase.camelize, camelize], ids=["c", "drf"])
def test_camelize_benchmark(benchmark, decamelized_dict, impl):
    benchmark(impl, decamelized_dict)


def test_camelize_list_of_dict(decamelized_list_of_dict, camelized_list_of_dict) -> None:
    got = snakecase.camelize(decamelized_list_of_dict)
    assert (
        got == camelized_list_of_dict
    ), f"{json.dumps(decamelized_list_of_dict, indent=4)} != {json.dumps(camelized_list_of_dict, indent=4)}"



from copy import deepcopy

import pytest


@pytest.fixture
def camelized_dict():
    return {"someObject": {"thatShould": "be", "deCamelized": [{"evenIf": "its nested inside list"}]}}


@pytest.fixture
def decamelized_dict():
    return {"some_object": {"that_should": "be", "de_camelized": [{"even_if": "its nested inside list"}]}}


@pytest.fixture
def decamelized_nested_dict():
    return deepcopy(
        {
            "id": 123,
            "username": "abc",
            "first_name": "def",
            "last_name": "ghi",
            "date_joined": "jkl",
            "last_login": "mno",
            "email": "p",
            "logged_in": True,
            "is_superuser": True,
            "is_staff": True,
            "permissions": ["q"],
            "metadata": {"filters": {"proposals": {}}, "eggs": [{"1": "rabbit"}, {"2": "snake"}]},
        }
    )


@pytest.fixture
def decamelized_list_of_dict(decamelized_nested_dict):
    return [decamelized_nested_dict]


@pytest.fixture
def camelized_nested_dict():
    return deepcopy(
        {
            "id": 123,
            "username": "abc",
            "firstName": "def",
            "lastName": "ghi",
            "dateJoined": "jkl",
            "lastLogin": "mno",
            "email": "p",
            "loggedIn": True,
            "isSuperuser": True,
            "isStaff": True,
            "permissions": ["q"],
            "metadata": {"filters": {"proposals": {}}, "eggs": [{"1": "rabbit"}, {"2": "snake"}]},
        }
    )


@pytest.fixture
def camelized_list_of_dict(camelized_nested_dict):
    return [camelized_nested_dict]



import re
from collections import OrderedDict

camelize_re = re.compile(r"[a-z0-9]?_[a-z0-9]")


def underscore_to_camel(match):
    group = match.group()
    if len(group) == 3:
        return group[0] + group[2].upper()
    else:
        return group[1].upper()


def camelize(data, **options):
    # Handle lazy translated strings.
    ignore_fields = options.get("ignore_fields") or ()
    if isinstance(data, dict):
        new_dict = OrderedDict()
        for key, value in data.items():
            if isinstance(key, str) and "_" in key:
                new_key = re.sub(camelize_re, underscore_to_camel, key)
            else:
                new_key = key
            if key not in ignore_fields and new_key not in ignore_fields:
                new_dict[new_key] = camelize(value, **options)
            else:
                new_dict[new_key] = value
        return new_dict
    if is_iterable(data) and not isinstance(data, str):
        return [camelize(item, **options) for item in data]
    return data


def get_underscoreize_re(options):
    if options.get("no_underscore_before_number"):
        pattern = r"([a-z0-9]|[A-Z]?(?=[A-Z](?=[a-z])))([A-Z])"
    else:
        pattern = r"([a-z0-9]|[A-Z]?(?=[A-Z0-9](?=[a-z0-9]|$)))([A-Z]|(?<=[a-z])[0-9](?=[0-9A-Z]|$)|(?<=[A-Z])[0-9](?=[0-9]|$))"
    return re.compile(pattern)


def camel_to_underscore(name, **options):
    underscoreize_re = get_underscoreize_re(options)
    return underscoreize_re.sub(r"\1_\2", name).lower()


def _get_iterable(data):
    return data.items()


def underscoreize(data, **options):
    if isinstance(data, dict):
        new_dict = {}
        for key, value in _get_iterable(data):
            if isinstance(key, str):
                new_key = camel_to_underscore(key, **options)
            else:
                new_key = key
            new_dict[new_key] = value
        return new_dict
    if is_iterable(data):
        return [underscoreize(item, **options) for item in data]

    return data


def is_iterable(obj):
    try:
        iter(obj)
    except TypeError:
        return False
    else:
        return True



import logging
import sys
from distutils.command.build_ext import build_ext
from distutils.core import Extension
from distutils.errors import CCompilerError
from distutils.errors import DistutilsExecError
from distutils.errors import DistutilsPlatformError

from setuptools import Extension
from setuptools import setup

log = logging.getLogger(__name__)

args = []
if sys.platform == "win32":
    args.append("/Ox")
else:
    args.append("-O3")

ext_modules = [
    Extension(
        "_case", sources=["src/yascc/case.c", "src/yascc/stack.c"], include_dirs=["src/yascc"], extra_compile_args=args
    )
]


class ExtBuilder(build_ext):
    def run(self):
        try:
            build_ext.run(self)
        except (DistutilsPlatformError, FileNotFoundError) as e:
            log.error("Could not compile C extension.")
            raise e from None

    def build_extension(self, ext):
        try:
            build_ext.build_extension(self, ext)
        except (
            CCompilerError,
            DistutilsExecError,
            DistutilsPlatformError,
            ValueError,
        ) as e:
            log.error("Could not compile C extension.")
            raise e from None


setup(ext_modules=ext_modules, cmdclass={"build_ext": ExtBuilder})
